#vider la mmoire
rm(list=ls())

####
# traitement des donnes individuelles
###

#importer les donnes orginales
#raw data
library(xlsx)
DRaw <- read.xlsx("titanic.xlsx",header=TRUE,sheetName="RAW")

#afficher les premires lignes
print(head(DRaw))

#Carac. de la base
str(DRaw)

#distribution par variable
print(summary(DRaw))

#rgression logistique avec ces donness
mRaw <- glm(SURVIVANT ~ CLASSE_ + AGE_ + SEXE_, data = DRaw, family = binomial)
print(summary(mRaw))

#nombre de combinaisons thoriques possibles
print(nlevels(DRaw$CLASSE)*nlevels(DRaw$AGE)*nlevels(DRaw$SEXE))

#calcul des effectifs selon les croisements
#nous constatons que certaines combinaisons n'existent pas
#on note surtout : pas d'enfants (AGE = CHILD) qui soient membres d'quipage (CLASSE = CREW)
print(table(DRaw[2:ncol(DRaw)]))

####
# traitement des donnes groupes
###

#chargement du second dataset
DPat <- read.xlsx("titanic.xlsx",header=TRUE,sheetName = "PATTERN")

#afficher la base
print(DPat)

#carac. de la base
str(DPat)

#nombre de Pattern
M <- nrow(DPat)
print(M)

#crer la matrice qui corresponde  la variable cible
response <- as.matrix(DPat[c("SURV_YES","SURV_NO")])
print(response)

#raliser la rgression sur donnes groupes
#response est une matrice  2 colonnes, 1ere nb de positifs, 2nde, nb de ngatifs
#coefficients estims et carts-type identiques
#deviance et degrs de libert diffrents
mPat <- glm(response ~ CLASSE_ + AGE_ + SEXE_, data = DPat, family = binomial)
print(summary(mPat))

#nombre de paramtres estims dans le modle
J <- length(mPat$coefficients)
print(J)

### LEVIER ###

#calcul du levier
h <- hatvalues(mPat)
print(h)

#vrifier la somme
print(sum(h))

#barplot
barplot(h,horiz=TRUE,las=2,main="Levier",cex.names=0.8)

#N effectif pour chaque pattern
N <- DPat$SURV_YES + DPat$SURV_NO
print(N)

#correspondance avec les donnes -- afficher les effectifs
print(data.frame(N=N,DPat$CLASSE,DPat$AGE,DPat$SEXE))

### RESIDU DE PEARSON ###

#colonne Y -- nombre de positifs pour chaque pattern
Y <- DPat$SURV_YES
print(Y)

#proba d'tre positif pour chaque pattern
proba <- predict(mPat,newdata=DPat,type="response")
print(proba)

#YHAT, estimation du nombre de positifs pour chaque pattern
YHAT <- N * proba
print(YHAT)

#rsidu de Pearson
res_pearson <- (Y - YHAT)/sqrt(N*proba*(1.0-proba))
print(res_pearson)

#avec la fonction de R
residuals(mPat,"pearson")

#statistique de Pearson
stat_pearson <- sum(res_pearson^2)
print(stat_pearson)

#proba. critique -- p-value
print(pchisq(stat_pearson,M-J,lower.tail = FALSE))

#rsidu standardis de Pearson
rstd_pearson <- rstandard(mPat,type="pearson")
print(rstd_pearson)

#contrib. Pearson
contrib_pearson <- rstd_pearson^2
print(contrib_pearson)

#graphique
barplot(contrib_pearson,horiz=TRUE,las=2,main="Contrib.Pearson",cex.names=0.8)

#revenir aux donnes - cart entre SURV=YES obs. et estim, effectif
#les profils n10 et n6 correspondent aux carac. enonces
print(data.frame(ECART=Y-YHAT,N=N,DPat$CLASSE,DPat$AGE,DPat$SEXE))

#Khi-deux de Pearson sans N10 et n6
khi2_pearson_bis <- stat_pearson - sum(contrib_pearson[c(6,10)])
print(khi2_pearson_bis)

#proba critique associe
#sans ces deux profils, la modlisation serait satisfaisante
print(pchisq(khi2_pearson_bis,(M-2)-J,lower.tail=FALSE))

### RESIDU DEVIANCE ###
      
#rsidu dviance
res_dev <- residuals(mPat,"deviance")
print(res_dev)

#stat deviance
stat_deviance <- sum(res_dev^2)
print(stat_deviance)

#p-value
print(pchisq(stat_deviance,M-J,lower.tail = FALSE))

#rsidu standardis deviance
rstd_dev <- rstandard(mPat,type="deviance")
print(rstd_dev)

#grahique
plot(1:M,rstd_dev,type="p",main="Res. standardiss Deviance")
abline(h=0,col="gray")
abline(h=-2,col="red",lty=2)
abline(h=+2,col="red",lty=2)

#contrib deviance
contrib_dev <- rstd_dev^2

#barplot
barplot(contrib_dev,horiz=TRUE,las=2,main="Contrib.Deviance",cex.names=0.8)

### DISTANCE DE COOK ###

#distance de Cook
dcook <- cooks.distance(mPat)
print(dcook)

#graphique
barplot(dcook,horiz=TRUE,las=2,main="Distance de Cook",cex.names=0.8)

### REG. AVEC INTERACTION (CLASSE * SEXE) ###

#rajouter l'interaction MALE et CLASSE
mPatBis <- glm(response ~ CLASSE_ * SEXE_ + AGE_, data = DPat, family = binomial)
print(summary(mPatBis))

#vrifier si le terme d'interaction est siginifcatif
#test du rapport de varisemblance
LR <- mPat$deviance - mPatBis$deviance
print(LR)

#p-value
print(pchisq(LR,mPat$df.residual-mPatBis$df.residual,lower.tail=FALSE))

#est-ce que pour autant le modle est compatible avec les donnes ?
#non, il manque des informations pour expliquer convenablement la survie
print(pchisq(mPatBis$deviance,mPatBis$df.residual,lower.tail=FALSE))
